@@ -1,6 +1,6 @@
 // test_wtree.cc
 //
-//   Copyright (C) 2005, 2008 Daniel Burrows
+//   Copyright (C) 2005, 2008-2010 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -21,10 +21,15 @@
 
 #include <generic/util/immset.h>
 
+#include <limits.h>
+
 using imm::map;
+using imm::nil_t;
 using imm::set;
 using imm::wtree_node;
 
+nil_t nil;
+
 template<typename T>
 std::ostream &operator<<(std::ostream &out, const imm::set<T> &t)
 {
@@ -81,9 +86,11 @@ class WTreeTest : public CppUnit::TestFixture
   CPPUNIT_TEST(testLessThan);
   CPPUNIT_TEST(testIntersects);
   CPPUNIT_TEST(testContains);
+  CPPUNIT_TEST(testAccumulation);
   CPPUNIT_TEST(generalWTreeTest);
   CPPUNIT_TEST(mapTest);
   CPPUNIT_TEST(mapIntersectTest);
+  CPPUNIT_TEST(setForEachBreakTest);
 
   CPPUNIT_TEST_SUITE_END();
 public:
@@ -93,8 +100,9 @@ public:
   void testRotateLeft()
   {
     int_node a(5,
-	       int_node(3, int_node(2), int_node(4)),
-	       int_node(7, int_node(6), int_node(8)));
+	       int_node(3, int_node(2, nil), int_node(4, nil), nil),
+	       int_node(7, int_node(6, nil), int_node(8, nil), nil),
+	       nil);
 
     int_node a_left = a.getLeft();
     int_node a_right = a.getRight();
@@ -128,7 +136,7 @@ public:
 
 
 
-    int_node b = a.left_rotate_single();
+    int_node b = a.left_rotate_single(nil);
 
     CPPUNIT_ASSERT_EQUAL(a.size(), b.size());
 
@@ -171,8 +179,9 @@ public:
   void testRotateRight()
   {
     int_node a(5,
-	       int_node(3, int_node(2), int_node(4)),
-	       int_node(7, int_node(6), int_node(8)));
+	       int_node(3, int_node(2, nil), int_node(4, nil), nil),
+	       int_node(7, int_node(6, nil), int_node(8, nil), nil),
+	       nil);
 
     int_node a_left = a.getLeft();
     int_node a_right = a.getRight();
@@ -206,7 +215,7 @@ public:
 
 
 
-    int_node c = a.right_rotate_single();
+    int_node c = a.right_rotate_single(nil);
 
     CPPUNIT_ASSERT_EQUAL(a.size(), c.size());
     CPPUNIT_ASSERT(c.isValid());
@@ -242,10 +251,13 @@ public:
 
   void testDoubleRotateLeft()
   {
-    int_node a = int_node(3, 2,
+    int_node a = int_node(3,
+			  int_node(2, nil),
 			  int_node(7,
-				   int_node(5, 4, 6),
-				   8));
+				   int_node(5, int_node(4, nil),
+					    int_node(6, nil), nil),
+				   int_node(8, nil), nil),
+			  nil);
 
     int_node a_left = a.getLeft(), a_right = a.getRight();
 
@@ -277,7 +289,7 @@ public:
     CPPUNIT_ASSERT_EQUAL(8, a_right_right.getVal());
 
 
-    int_node b = a.left_rotate_double();
+    int_node b = a.left_rotate_double(nil);
 
     CPPUNIT_ASSERT(b.isValid());
 
@@ -316,8 +328,13 @@ public:
   void testDoubleRotateRight()
   {
     int_node a = int_node(7,
-			  int_node(3, 2, int_node(5, 4, 6)),
-			  8);
+			  int_node(3,
+				   int_node(2, nil),
+				   int_node(5,
+					    int_node(4, nil),
+					    int_node(6, nil), nil),
+				   nil),
+			  int_node(8, nil), nil);
 
     CPPUNIT_ASSERT(a.isValid());
 
@@ -351,7 +368,7 @@ public:
 
 
 
-    int_node b = a.right_rotate_double();
+    int_node b = a.right_rotate_double(nil);
 
     CPPUNIT_ASSERT(b.isValid());
 
@@ -748,6 +765,64 @@ do  { \
     assertNotContains(t, s3);
   }
 
+  class maxAccumOps
+  {
+  public:
+    int empty() const { return INT_MIN; }
+    int project(const std::pair<int, int> &p) const { return p.first; }
+    int merge(int n, int m) const { return std::max<int>(n, m); }
+  };
+
+  void testAccumulation()
+  {
+    // Build a set of pairs where we accumulate the maximum of the
+    // first element in the pair.
+    set<std::pair<int, int>,
+        aptitude::util::compare3_f<std::pair<int, int> >,
+        int,
+        maxAccumOps> s;
+
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), INT_MIN);
+
+    s.insert(std::make_pair(5, 20));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 5);
+
+    s.insert(std::make_pair(10, 5));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 10);
+
+    s.insert(std::make_pair(5, 10));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 10);
+
+    s.insert(std::make_pair(-1, 4));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 10);
+
+    s.insert(std::make_pair(1, 6));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 10);
+
+    s.erase(std::make_pair(10, 5));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 5);
+
+    s.erase(std::make_pair(5, 20));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 5);
+
+    s.insert(std::make_pair(2, 4));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 5);
+
+    s.erase(std::make_pair(5, 10));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 2);
+
+    s.erase(std::make_pair(2, 4));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), 1);
+
+    s.erase(std::make_pair(1, 6));
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), -1);
+
+    s.erase(std::make_pair(-1, 4));
+    CPPUNIT_ASSERT_EQUAL((unsigned)0, s.size());
+    CPPUNIT_ASSERT(s.empty());
+    CPPUNIT_ASSERT_EQUAL(s.getAccumVal(), INT_MIN);
+  }
+
 #define assertNotSupermap(a, b) \
   do { \
     if(a.is_supermap_of(b)) \
@@ -1178,6 +1253,64 @@ do  { \
     CPPUNIT_ASSERT(!m2.has_related_mapping(m1, std::greater<std::pair<int, int> >()));
     CPPUNIT_ASSERT(m2.has_related_mapping(m1, std::less<std::pair<int, int> >()));
   }
+
+  // Set the Nth entry in the given array to true, until the entry
+  // number is above 5.
+  struct set_or_break
+  {
+    bool *array;
+
+    set_or_break(bool *_array) : array(_array)
+    {
+    }
+
+    bool operator()(int n) const
+    {
+      array[n] = true;
+
+      return n <= 5;
+    }
+  };
+
+  void setForEachBreakTest()
+  {
+    const int array_len = 20;
+    bool array[array_len];
+
+    for(int i = 0; i < array_len; ++i)
+      array[i] = false;
+    {
+      imm::set<int> vals;
+
+      vals.insert(2);
+      CPPUNIT_ASSERT(vals.for_each(set_or_break(array)));
+
+      for(int i = 0; i < array_len; ++i)
+	{
+	  CPPUNIT_ASSERT_EQUAL(vals.contains(i), array[i]);
+	}
+    }
+
+
+    for(int i = 0; i < array_len; ++i)
+      array[i] = false;
+    {
+      imm::set<int> vals;
+
+      vals.insert(2);
+      vals.insert(4);
+      vals.insert(5);
+      vals.insert(6);
+      vals.insert(8);
+      CPPUNIT_ASSERT(!vals.for_each(set_or_break(array)));
+
+      // Now everything *up to and including* 6 should be set.
+      for(int i = 0; i < array_len; ++i)
+	{
+	  CPPUNIT_ASSERT_EQUAL(vals.contains(i) && i <= 6, array[i]);
+	}
+    }
+  }
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(WTreeTest);
